// ignore_for_file: non_constant_identifier_names, unnecessary_brace_in_string_interps, unused_local_variable

import 'dart:convert';

import 'package:general_lib/general_lib.dart';
import 'package:mason_logger/mason_logger.dart';
import 'package:path/path.dart';
import 'package:telegram_bot_template/database/database_core.dart';
import 'package:telegram_bot_template/database/database_type.dart';
import 'package:telegram_bot_template/scheme/config_runtime.dart';
import 'package:telegram_bot_template/scheme/tg_client_data.dart';
import 'package:telegram_bot_template/src/update_callback_query.dart';
import 'package:telegram_bot_template/src/update_inline_query.dart';
import 'package:telegram_bot_template/src/update_message.dart';
import 'package:telegram_client/telegram_client.dart';
import 'dart:io/io.dart';

void main(List<String> args) async {
  Logger logger = Logger(level: Level.verbose);
  logger.info("Telegram Client");
  Directory directory_tg = Directory(join(Directory.current.path, "tg"));
  if (!directory_tg.existsSync()) {
    await directory_tg.create(recursive: true);
  }

  Directory tg_bot_db_dir = Directory(join(directory_tg.path, "db"));
  if (!tg_bot_db_dir.existsSync()) {
    await tg_bot_db_dir.create(recursive: true);
  }

  String token_bot = "";
  Directory database_directory = Directory(join(directory_tg.path, "client"));

  ConfigRuntime configRuntime = ConfigRuntime({});

  DatabaseLib databaseLib = DatabaseLib(
    directory: tg_bot_db_dir,
  );

  bool is_use_tdlib = configRuntime.environment.is_use_tdlib ?? false;
  print((is_use_tdlib) ? "Tdlib: IYA" : "TDLIB: NO");
  await databaseLib.init();

  // Box db = await Hive.openBox("galaxeus");
  Database database = Database(
    database_type: (configRuntime.database.database_type == "supabase")
        ? DatabaseType.supabase
        : DatabaseType.local,
    botType: BotType.glx,
    database_lib: databaseLib,
    configRuntime: configRuntime,
    is_userbot: false,
    systemInfoFetch: () async {
      String message_data = "System Info";
      try {
        Map res = (json.decode((await Process.run(
          "neofetch",
          [
            "--json",
          ],
        ))
            .stdout as String) as Map);

        message_data = jsonToMessage(res, jsonFullMedia: {});
      } catch (e) {}

      return message_data;
    },
  );

  TelegramClient tg = TelegramClient();

  tg.on(
    event_name: tg.event_update,
    onUpdate: (UpdateTelegramClient updateTelegramClient) async {
      try {
        await tg.autoSetData(updateTelegramClient);

        Map? update = await updateTelegramClient.update;
        if (update == null) {
          return null;
        }

        if (update["@type"] == "updateAuthorizationState") {
          if (update["authorization_state"] is Map) {
            Map authorization_state = update["authorization_state"];
            if (authorization_state["@type"] ==
                "authorizationStateWaitPhoneNumber") {
              Map res = await tg.invoke(
                parameters: {
                  "@type": "checkAuthenticationBotToken",
                  "token": token_bot,
                },
                telegramClientData: updateTelegramClient.telegramClientData,
              );

              if (res["@type"] == "error") {
                print(
                    "token bot error / salah silahkan isi token bot dengan benar ya");
                exit(0);
              }
              print(res.toStringifyPretty(2));
            }

            if (authorization_state["@type"] == "authorizationStateReady") {
              Map get_me = (await tg.request(
                parameters: {"@type": "getMe"},
                telegramClientData: updateTelegramClient.telegramClientData,
              ))["result"];
              get_me.removeByKeys(["phone_number"]);
              get_me.printPretty(2);
            }
            return null;
          }
          update.printPretty(2);
          return null;
        }

        TgClientData tg_client_data =
            TgClientData(updateTelegramClient.tgClientData());

        if (update["callback_query"] is Map) {
          return await callbackQuery(
            update["callback_query"],
            updateTelegramClient: updateTelegramClient,
            tgClientData: tg_client_data,
            tg: tg,
            database: database,
          );
        }

        if (update["inline_query"] is Map) {
          return await inlineQuery(
            update["inline_query"],
            updateTelegramClient: updateTelegramClient,
            tgClientData: tg_client_data,
            tg: tg,
            database: database,
          );
        }
        if (update["channel_post"] is Map) {
          return await updateMessage(
            update["channel_post"],
            updateTelegramClient: updateTelegramClient,
            tgClientData: tg_client_data,
            tg: tg,
            database: database,
          );
        }
        if (update["message"] is Map) {
          return await updateMessage(
            update["message"],
            updateTelegramClient: updateTelegramClient,
            tgClientData: tg_client_data,
            tg: tg,
            database: database,
          );
        }
      } catch (e, stack) {
        logger.err("${e} ${stack}");
      }
    },
    onError: (error, stackTrace) {},
  );

  await tg.init(
    pathTdlib: TgUtils.pathTdlib(),
    is_init_tdlib: (is_use_tdlib),
    telegramClientTdlibOption: TelegramClientTdlibOption(
      clientOption: {
        "database_directory": database_directory.path,
        'files_directory': directory_tg.path,
      },
    ),
  );
  if (is_use_tdlib) {
    await tg.tdlib.initIsolate();
  }
}
